﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using Telerik.Windows.Controls;
using Telerik.Windows.DragDrop;

namespace Drag_Drop_NavigationView
{
    public class NavigationViewDragDropBehavior
    {
        public double TreeViewItemHeight { get; set; }
        private bool isTreeSource = false;

        private RadNavigationView _associatedObject;
        public RadNavigationView AssociatedObject
        {
            get => _associatedObject;
            set => _associatedObject = value;
        }

        private static Dictionary<RadNavigationView, NavigationViewDragDropBehavior> instances;

        static NavigationViewDragDropBehavior()
        {
            instances = new Dictionary<RadNavigationView, NavigationViewDragDropBehavior>();
        }

        public static bool GetIsEnabled(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsEnabledProperty);
        }

        public static void SetIsEnabled(DependencyObject obj, bool value)
        {
            NavigationViewDragDropBehavior behavior = GetAttachedBehavior(obj as RadNavigationView);

            behavior.AssociatedObject = obj as RadNavigationView;

            if (value)
            {
                behavior.Initialize();
            }
            else
            {
                behavior.CleanUp();
            }
            obj.SetValue(IsEnabledProperty, value);
        }

        // Using a DependencyProperty as the backing store for IsEnabled.  This enables animation, styling, binding, etc... 
        public static readonly DependencyProperty IsEnabledProperty =
            DependencyProperty.RegisterAttached("IsEnabled", typeof(bool), typeof(NavigationViewDragDropBehavior),
                new PropertyMetadata(new PropertyChangedCallback(OnIsEnabledPropertyChanged)));

        public static void OnIsEnabledPropertyChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            SetIsEnabled(dependencyObject, (bool)e.NewValue);
        }

        private static NavigationViewDragDropBehavior GetAttachedBehavior(RadNavigationView gridview)
        {
            if (!instances.ContainsKey(gridview))
            {
                instances[gridview] = new NavigationViewDragDropBehavior
                {
                    AssociatedObject = gridview
                };
            }

            return instances[gridview];
        }

        protected virtual void Initialize()
        {
            DragDropManager.AddDragInitializeHandler(AssociatedObject, OnDragInitialize);
            DragDropManager.AddGiveFeedbackHandler(AssociatedObject, OnGiveFeedback);
            DragDropManager.AddDragDropCompletedHandler(AssociatedObject, OnDragDropCompleted);
            DragDropManager.AddDropHandler(AssociatedObject, OnDrop);
            TreeViewItemHeight = 24.0;
            AssociatedObject.Loaded += AssociatedObject_Loaded;

            
        }

        private void AssociatedObject_Loaded(object sender, RoutedEventArgs e)
        {
            foreach (var item in AssociatedObject.Items)
            {
                DragDropManager.RemoveDragOverHandler(item as UIElement, OnItemDragOver);
                DragDropManager.AddDragOverHandler(item as UIElement, OnItemDragOver);
            }
        }

        protected virtual void CleanUp()
        {
            DragDropManager.RemoveDragInitializeHandler(AssociatedObject, OnDragInitialize);
            DragDropManager.RemoveGiveFeedbackHandler(AssociatedObject, OnGiveFeedback);
            DragDropManager.RemoveDragDropCompletedHandler(AssociatedObject, OnDragDropCompleted);
            DragDropManager.RemoveDropHandler(AssociatedObject, OnDrop);
        }

        private IList sourceItems = null;
        private IList destinationItems = null;
        private object sourceItem = null;

        private void OnDragInitialize(object sender, DragInitializeEventArgs e)
        {
            RadNavigationViewItem navigationViewItem = e.OriginalSource as RadNavigationViewItem ?? (e.OriginalSource as FrameworkElement).ParentOfType<RadNavigationViewItem>();
            //object data = treeViewItem != null ? treeViewItem.Item : (sender as RadNavigationView).SelectedItem;
            Telerik.Windows.DragDrop.Behaviors.IDragPayload payload = DragDropPayloadManager.GeneratePayload(null);
            DropIndicationDetails dropDetails = new DropIndicationDetails
            {
                CurrentDraggedItem = navigationViewItem
            };
            DragVisual visual = new DragVisual()
            {
                Content = dropDetails,
                ContentTemplate = AssociatedObject.Resources["ItemDragTemplate"] as DataTemplate
            };
            payload.SetData("DraggedItem", navigationViewItem);
            payload.SetData("DropDetails", dropDetails);
            e.DragVisual = visual;
            e.DragVisualOffset = e.RelativeStartPoint;
            e.Data = payload;
            e.AllowedEffects = DragDropEffects.All;
            FrameworkElement sourceItem = e.OriginalSource as RadNavigationViewItem ?? (e.OriginalSource as FrameworkElement).ParentOfType<RadNavigationViewItem>();
            if (sourceItem == null)
            {
                sourceItems = (IList)AssociatedObject.ItemsSource;
            }
            else
            {
                sourceItems = AssociatedObject.Items;//(sourceItem as RadNavigationViewItem).Items != null ?
                   // (IList)(sourceItem as RadNavigationViewItem).Items : (IList)AssociatedObject.ItemsSource;
            }
            this.sourceItem = sourceItem.DataContext;
            destinationItems = AssociatedObject.Items as IList;
            isTreeSource = true;
        }

        private void OnGiveFeedback(object sender, Telerik.Windows.DragDrop.GiveFeedbackEventArgs e)
        {
            e.SetCursor(Cursors.Arrow);
            e.Handled = true;
        }

        private void OnDragDropCompleted(object sender, DragDropCompletedEventArgs e)
        {
            if (e.Effects != DragDropEffects.None && isTreeSource)
            {
                object data = DragDropPayloadManager.GetDataFromObject(e.Data, "DraggedItem");
                sourceItems.Remove(data);
            }
        }
        private void OnDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
        {
            if (isTreeSource)
            {
                object data = DragDropPayloadManager.GetDataFromObject(e.Data, "DraggedItem");

                sourceItems.Remove(data);
            }
            if (e.Effects != DragDropEffects.None)
            {
                RadNavigationViewItem destinationItem = (e.OriginalSource as FrameworkElement).ParentOfType<RadNavigationViewItem>();
                DropIndicationDetails dropDetails = DragDropPayloadManager.GetDataFromObject(e.Data, "DropDetails") as DropIndicationDetails;
                object data = DragDropPayloadManager.GetDataFromObject(e.Data, "DraggedItem");

                if (destinationItems != null)
                {
                    int dropIndex = dropDetails.DropIndex >= destinationItems.Count ? destinationItems.Count :
                        dropDetails.DropIndex < 0 ? 0 : dropDetails.DropIndex;

                    destinationItems.Insert(dropIndex, data);
                }
            }
            IEnumerable source = AssociatedObject.ItemsSource;
            AssociatedObject.ItemsSource = null;
            AssociatedObject.ItemsSource = source;
            if (isTreeSource)
            {
                isTreeSource = false;
                sourceItem = null;
                sourceItems = null;
                destinationItems = null;
            }
        }

        private void OnItemDragOver(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
        {
            object draggedData = DragDropPayloadManager.GetDataFromObject(e.Data, "DraggedItem");
            DropIndicationDetails dropDetails = DragDropPayloadManager.GetDataFromObject(e.Data, "DropDetails") as DropIndicationDetails;
            RadNavigationViewItem item = (e.OriginalSource as FrameworkElement).ParentOfType<RadNavigationViewItem>();

            DropPosition position = GetPosition(item, e.GetPosition(item));
            if (position != DropPosition.Inside)
            {
                e.Effects = DragDropEffects.All;
                destinationItems = (IList)AssociatedObject.Items;
                int index = destinationItems.IndexOf(item);
                dropDetails.DropIndex = position == DropPosition.Before ? index : index + 1;
            }
            else
            {
                destinationItems = (IList)item.ItemsSource;
                int index = 0;

                if (destinationItems == null)
                {
                    e.Effects = DragDropEffects.None;
                }
                else
                {
                    e.Effects = DragDropEffects.All;
                    dropDetails.DropIndex = index;
                }
            }
            dropDetails.CurrentDraggedOverItem = item;
            dropDetails.CurrentDropPosition = position;
            if (isTreeSource && IsChildOfSource(item))
            {
                e.Effects = DragDropEffects.None;
            }
            e.Handled = true;
        }

        private bool IsChildOfSource(FrameworkElement item)
        {
            FrameworkElement currentItem = item;

            //while (currentItem != null)
            //{
            //    if ((currentItem as RadNavigationViewItem).Item == sourceItem)
            //    {
            //        return true;
            //    }

            //    currentItem = currentItem.ParentOfType<RadNavigationViewItem>();
            //}
            return false;
        }

        private DropPosition GetPosition(RadNavigationViewItem item, Point point)
        {
            if (point.Y < TreeViewItemHeight / 4)
            {
                return DropPosition.Before;
            }
            else if (point.Y > TreeViewItemHeight * 3 / 4)
            {
                return DropPosition.After;
            }
            return DropPosition.Inside;
        }
    }
}
